--[[
    Copyright (C) 2011 Omer Bahri Gordebak
    Using some of the code provided by:
    Copyright (c) 2010 Enrique García Cota

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Omer Bahri Gordebak <gordebak@gmail.com>
]]
level = 1
ballx = 0
bally = 0
ball2x = 0
ball2y = 0
exitx = 0
exity = 0
reversedx = -64
reversedy = -64
reversed = false
isfullscreen = false
ismusic = true
isfx = true
issplash = true
isfinished = false
musicoffx = -60
fxoffx = -60

local tileW, tileH, tileset, quads, tileTable

function love.load()
  love.window.setMode( 640, 480, {fullscreen=false, fullscreentype="desktop"} )
  love.window.setTitle( "Semicontrol" )
  image = love.graphics.newImage("images/player.png")
  image2 = love.graphics.newImage("images/player2.png")
  wall = love.graphics.newImage("images/wall.png")
  exit = love.graphics.newImage("images/exit.png")
  image3 = love.graphics.newImage("images/playerreversed.png")
  reverse = love.graphics.newImage("images/reverse.png")
  splash = love.graphics.newImage("images/splash.png")
  finished = love.graphics.newImage("images/finish.png")
  musicoff = love.graphics.newImage("images/off.png")
  fxoff = love.graphics.newImage("images/off.png")
  levelup = love.audio.newSource("sounds/levelup.wav")
  music = love.audio.newSource("sounds/sq-never-satisfied-music.wav")
  music:setLooping(true)
  love.audio.play(music)

  files = love.filesystem.getDirectoryItems("maps")
  levelcount = 0
  for _,level in ipairs(files) do
      levelcount = levelcount + 1
  end
  maps = {}
  for _,m in ipairs(files) do table.insert(maps, "maps/"..m) end
  table.sort(maps)
  loadMap(maps[level])
end

function love.mousepressed(x, y)
if isfullscreen == true then
   sf = love.graphics.getHeight() / 480
else
   sf = 1
end
if isfinished == false then
   if x > 60*sf and x < 300*sf and y > 150*sf and y < 280*sf and issplash == true then
       issplash = false
   end
   if x > 90*sf and x < 270*sf and y > 270*sf and y < 320*sf and issplash == true then
      if ismusic == true then
         music:stop()
         ismusic = false
      else
         love.audio.play(music)
         ismusic = true
      end
   end
   if x > 90*sf and x < 270*sf and y > 330*sf and y < 380*sf and issplash == true then
         if isfx == true then
              isfx = false
         else
              isfx = true
         end
   end
   end
end

function love.keypressed(key)
   if key == "escape" and issplash == true then
      love.event.push("quit")
   end
   if key == "escape" and issplash == false then
      if isfinished == false then
        issplash = true
      else
        love.event.quit()
      end
   end
   if key == "space" and issplash == false and isfinished == false then
      level = level + 1
      if level > levelcount then
          level = 1
      end
      loadMap(maps[level])
   end
   if key == "r" then
      love.event.quit("restart")
   end
   if key == "space" and issplash == true and isfinished == false then
      issplash = false
      loadMap(maps[level])
   end
   if key == "f5" and issplash == false and isfinished == false then
      loadMap(maps[level])
   end
   if key == "m" then
      if ismusic == true then
          music:stop()
          ismusic = false
      else
          love.audio.play(music)
          ismusic = true
      end
   end
   if key == "f" then
      if isfx == true then
          isfx = false
      else
          isfx = true
      end
   end
   if key == "f11" then
      if isfullscreen == false then
         love.window.setMode(640, 480, {fullscreen=true, fullscreentype="desktop"})
         isfullscreen = true
      else
         love.window.setMode(640, 480, {fullscreen=false, fullscreentype="desktop"})
         isfullscreen = false
      end
   end
   if key == "left" then
      go(-32, 0) 
   end
   if key == "right" then
      go(32, 0)
   end
   if key == "up" then
      go(0, -32)
   end
   if key == "down" then
      go(0, 32)
   end
end

function love.draw()
  love.mouse.setVisible(false)
  if isfullscreen == true then
    if issplash == false then
        screen_width = love.graphics.getWidth()
        love.graphics.scale(screen_width / 608)
    else
        screen_height = love.graphics.getHeight()
        love.graphics.scale(screen_height / 480)
    end
  else
    screen_width = love.graphics.getWidth()
    love.graphics.scale(screen_width / 608)
  end
  drawMap()
  love.graphics.draw(image, ballx, bally)
  love.graphics.draw(reverse, reversedx, reversedy)
  if reversed == true then
    love.graphics.draw(image2, ball2x, ball2y)
    love.graphics.draw(image3, ball2x, ball2y)
    love.graphics.setColor(0,0,0)
    love.graphics.circle("fill", ball2x + 15, ball2y + 15, 5)
    love.graphics.setColor(255,255,255)
  else
    love.graphics.draw(image3, ball2x, ball2y)
    love.graphics.draw(image2, ball2x, ball2y)
  end
  love.graphics.draw(exit, exitx, exity)
  if issplash == true then
      love.mouse.setVisible(true)
      love.graphics.draw(splash, 0, 0)
      if ismusic == true then
          musicoffx = -60
      else
          musicoffx = 190
      end
      if isfx == true then
          fxoffx = -60
      else
          fxoffx = 190
      end
      love.graphics.draw(musicoff, musicoffx, 280)
      love.graphics.draw(fxoff, fxoffx, 344)
  end
  if isfinished == true then
      love.graphics.draw(finished, 0, 0)
  end
end

function loadMap(path)
  love.filesystem.load(path)() -- attention! extra parenthesis
end

function newMap(tileString)
love.graphics.clear()
velx, vely = 0, 0
tilecorners = {}
exitx,exity = 0,0
ballx,bally = 0,0
ball2x,ball2y = 0,0
reversedx,reversedy = -64,-64
reversed = false
cb = 0

  tileTable = {}

  local width = #(tileString:match("[^\n]+"))

  for x = 1,width,1 do tileTable[x] = {} end

  local rowIndex,columnIndex = 1,1
  for row in tileString:gmatch("[^\n]+") do
    assert(#row == width, 'Map is not aligned: width of row ' .. tostring(rowIndex) .. ' should be ' .. tostring(width) .. ', but it is ' .. tostring(#row))
    columnIndex = 1
    for character in row:gmatch(".") do
      tileTable[columnIndex][rowIndex] = character
	if character == "#" then
	  cb = cb + 1
	  tilecorners[cb] = { (columnIndex - 1)*32 + 32, (rowIndex - 1)*32 + 32, rowIndex, columnIndex }
	end
	if character == "X" then
	  exitx = (columnIndex - 1)*32 + 32
	  exity = (rowIndex - 1)*32 + 32
	end
    if character == "1" then
      ballx = (columnIndex - 1)*32 + 32
      bally = (rowIndex - 1)*32 + 32
    end
    if character == "2" then
      ball2x = (columnIndex - 1)*32 + 32
      ball2y = (rowIndex - 1)*32 + 32
    end
    if character == "R" then
      reversedx = (columnIndex - 1)*32 + 32
      reversedy = (rowIndex - 1)*32 + 32
    end
      columnIndex = columnIndex + 1
    end
    rowIndex=rowIndex+1
  end
end

function drawMap()
  for _,tiletable in ipairs(tilecorners) do
      local x,y = tiletable[1], tiletable[2]
      love.graphics.draw(wall, x, y) 
  end
end

function go(move1x, move1y)
   local move2x, move2y = 0, 0
   if reversed == false then
      move2x = move1x
      move2y = move1y
   else
      move2x = -move1x
      move2y = -move1y
   end
      local move = true
      ballx = ballx + move1x
      bally = bally + move1y
      ball2x = ball2x + move2x
      ball2y = ball2y + move2y
      for _,tiletable in ipairs(tilecorners) do
         local x,y = tiletable[1],tiletable[2]
         if ballx == x and bally == y then
             ballx = ballx - move1x
             bally = bally - move1y
             ball2x = ball2x - move2x
             ball2y = ball2y - move2y
             move = false
         end
      end
      for _,tiletable in ipairs(tilecorners) do
         local x,y = tiletable[1],tiletable[2]
         if ball2x == x and ball2y == y and move == true then
             ball2x = ball2x - move2x
             ball2y = ball2y - move2y
         end
      end
       if ball2x == exitx and ball2y == exity then
          level = level + 1
          if isfx == true then
            love.audio.play(levelup)
          end
          if level > levelcount then
             isfinished = true
          else
            loadMap(maps[level])
          end
        end
        if ball2x == reversedx and ball2y == reversedy then
           reversed = true;
        end
end
